//==============================================================================
// 1raster-tracker music converter application (2013 Hermit Software)
//==============================================================================
//Converts from MIDI to ORM (1raster-tracker) format,
//or converts from ORM to SWM (SID-Wizard 1.x) format.
//Additionally can make .sid out of .S00 (Vice sequential file)

//-----------------source-library inclusions------------------------------------
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <math.h>

//-----------------------literal constant definitions---------------------------
#define FILENAME_LENGTH_MAX 256
#define C64_FILENAME_LENGTH_MAX 16
#define TUNEDATA_SIZE_MAX 65536
#define EXT_LENGTH_MAX 4
#define TYPE_ID_LENGTH_MAX 32
#define MIDI_CHANNELS_MAX 64  //maximum allowed number of MIDI channels (tracks in MIDI format 1)
#define MIDI_TRACKLEN_MAX 65536 //maximum length of one MIDI-track
#define MIDI_VLV_LEN_MAX 8 //maximum length of variable-length variables in MIDI
#define SID_CHNAMOUNT 3   //SID/SWM maximal amount of channels - hardwired
#define VICE_P00_OFFSET 0x1A //vice program-offset (not containing PRG load-address following after) 
#define XM_COL_PER_TRK 5 //number of (byte) columns in one XM track (note,inst.,vol.,fx,fx-value
#define XM_GATEOFF_VALUE 97
#define XM_PATTERNS_MAX 256
#define XM_INSTRUMENTS_MAX 128 
#define XM_INSNAME_SIZE 22
 char* SWM_ID="SWM1";
 
//ORM format descriptors
#define ORM_HEADER_OFFSET 0x102
#define ORM_HEADER_SIZE 0x40
#define ORM_PATTERN_LENGTH 0x20
#define ORM_MAX_PATTERN_AMOUNT 8
#define ORM_SEQUENCE_LENGTH 0x40
unsigned char ORM_ID[]={0x4C,0xA2,0x10,0xA5,0};
unsigned char ORM_HEADER_SAMPLE[]={ 0x08,0x21,0xF5, 0x07,0x41,0xC5, 0x06,0x00,0xF9, 0x08,0xF1,0x1F, 0x08,0x01,0x1F, 1,1, ORM_PATTERN_LENGTH-1,1, 0,0,0 };
#define ORM_AUTHORINFO_OFFSET ORM_HEADER_SIZE-ORM_AUTHORINFO_SIZE
#define ORM_AUTHORINFO_SIZE 40
unsigned char ORM_AUTHORINFO_SAMPLE[] = "AUTHORNAME : TUNE-TITLE...              ";
#define ORM_TEMPO_OFFSET 1
#define ORM_ORDERLIST_OFFSET ORM_HEADER_OFFSET+ORM_SEQUENCE_LENGTH

//MIDI format descriptor constants======================================
//Header-chunk (every value is in big-endian format)
#define MIDI_ID_OFFSET 0
char* MIDI_ID="MThd";
#define MIDI_ID_SIZE 4
#define MIDI_HEADERSIZE_OFFSET 4 //the header-size of the MIDI is a big-endian double-word 
unsigned char MIDI_HEADERSIZE[4]={0,0,0,0x6}; //always 00 00 00 06 for standard MIDI files
#define MIDI_VERSION_OFFSET 8  //big endian 2-byte word
unsigned int InputMIDIver; //(0:single-track,1:multiple-track,2:multiple song)
#define MIDI_TRACK_AMOUNT_OFFSET 0xA //2 byte big-endian word containing number of tracks
unsigned int MIDI_TRACK_AMOUNT;
#define MIDI_DIVISION_OFFSET 0xC //2 byte - If the value is positive, then it represents the units per beat. For example, +96 would mean 96 ticks per beat.
signed int MIDI_DIVISION;
#define MIDI_TRACKS_OFFSET 0xE //here tracks are starting
 //Track-chunk - offset values here are relative to track-chunks' beginnings
 #define MIDI_TRACK_ID_OFFSET 0
 char* MIDI_TRACK_ID="MTrk"; //4 bytes - the literal string MTrk. This marks the beginning of a track. 
 #define MIDI_TRACK_SIZE_OFFSET 4 //4 bytes - the number of bytes in the track chunk following this number. 
 unsigned long MIDI_TRACK_SIZE[64]; //collecting track-sizes here
 unsigned long MIDI_TRACK_OFFSET[64]; //calculating from sizes
 #define MIDI_TRACK_EVENT_OFFSET 8 //a sequenced track event
  //Track-event format (consists of a delta time since the last event, and one of three types of events.)
  #define MIDI_DELTA_TIME_OFFSET 0 //variable-length big-endian value, last databyte has bit7 cleared (others have it set)
  #define MIDI_EVENT_OFFSET 0
  #define MIDI_META_EVENT_OFFSET 0
  #define MIDI_SYSEX_EVENT_OFFSET 0
   //Meta-event format
   #define MIDI_META_TYPE_OFFSET 0
   #define MIDI_META_LENGTH_OFFSET 1 //length of meta event data expressed as a variable length value. 
   #define MIDI_META_EVENT_DATA_OFFSET 0
   //System-exclusive event format
    //A system exclusive event can take one of two forms:
    //sysex_event = 0xF0 + <data_bytes> 0xF7 or sysex_event = 0xF7 + <data_bytes> 0xF7
    //In the first case, the resultant MIDI data stream would include the 0xF0. In the second case the 0xF0 is omitted.  


;//===========SID-Wizard Module Specification Description ====================
;// used by both 64tass and gcc source-files and compilation processes
;// $Id: SWM-spec.src 159 2012-10-30 00:44:08Z hermitsoft $
;//maximum amount of SWM tune-elements (patterns, instruments, etc.)
#define maxsubtuneamount 16-1 //max. $1F! - amount of subtunes ;counted from 0, so e.g 20 means 21 subtunes (0..$14) 
#define maxptnamount 99 //maximum number of patterns ;will depend on how much pattern-memory must be stolen for code
#define maxinstamount 48+1 //maximum number of instruments
#define seqlength 126 //$7e ;max. $7e - max. storage lenth of a sequence in orderlist
#define seqbound 128 //seqlength+2 ;max.$80 - sequence-boundary size - leave a little space at end of sequence for possible jumpaddress and $00
#define maxptnlen 250-1 //max $f9 (249) ;max size of pattern (in bytes) before any insertion is possible - max $f9 ($f8 effective rows + 4 byte with bigFX)
#define PTNBOUND 256 //maxptnlen+7 (max 256) ;LIKELY TO BE $100, but not more. Less doesn't utilize memory well...
#define maxinstsize 128  //$80 ;//maximum memory available for one specific instrument
#define instnamelength 8 //DON'T MODIFY!!! (will be incompatible) - maximum length of instruments' name - cropped at final tune-packing
#define MAXCHORDAMOUNT 64 //$40 ;//max. $40! - MAXIMUM NUMBER OF CHORDS FOR EMPTYZAK (INCLUDING DUMMYES AT BEGINNING AND END)
#define maxchordlength 32 //$20 ;//maximum length of a particular chord
#define MAXTEMPOPRAMOUNT 64 //$40 ;max. $40! - MAXIMUM NUMBER OF TEMPOPROGRAMS FOR EMPTYZAK (INCLUDING DUMMYES AT BEGINNING AND END)
#define maxtempolength 32 //$20 ;//maximum length of a particular tempo-program

#define DEFAULTPTNLEN 32 //$20 ;//FOR EMPTYZAK MACROS - used in editor/converter to define default length of yet unused patterns

;//PATTERN-EFFECT values =======================================================
;//note-column effect-values
;//NOT TO MODIFY THESE, HARDWIRED VALUES, POSSIBLY UNCHANGEABLE AND OCCURING MANY TIMES IN THE WHOLE CODE!
#define VIBRATOFX 96 //$60 ;//BASE OF VIBRATO-EFFECT
#define PACKEDMIN 112 //$70 ;//minimum value representing 2 packed NOPs
#define PACKEDMAX 119 //$77 ;//maximum value representing 9 packed NOPs
#define PORTAMFX 120 //$78 ;//AUTO-PORTAMENTO NOTE-EFFECT VALUE
#define DEFAULTPORTA 110 //AUTO-PORTAMENTO NOTE-FX DEFAULT SPEED
#define SYNCONFX 121 //$79 ;//SYN-ON NOTE-EFFECT VALUE
#define SYNCOFFX 122 //$7A ;//SYNC-OFF NOTE EFFECT VALUE
#define RINGONFX 123 //$7B ;//RING ON NOTE EFFECT VALUE
#define RINGOFFX 124 //$7C ;//RING-OFF NOTE EFFECT VALUE
#define GATEONFX 125 //$7D ;//GATE-ON (NOTESTART) NOTE EFFECT VALUE
#define GATEOFFX 126 //$7E ;//GATE-OFF(NOTE-MUTE) NOTE EFFECT VALUE

;//instrument/smallfx-column special effect-values
#define SWM_VOLUME_SMALLFX 80 //$50 - used to check in converter, if volume-column of XM should be used
#define SWM_MAIN_VOLUME_SMALLFX 160 //$A0 - used for converter
#define SWM_LEGATO_INSFX 63 //$3F - used to check in converter, if 3FF (max.portamento) fx should be used
#define SWM_VIBRATO_BIGFX 8 //used in converter to sustain NOTE-column-vibrato
#define SWM_DETUNE_FX 13 //$0D - used in converter
#define SWM_NOTEDELAY_FX 30 //$1e - used in converter
;//further effects are described in the user manual (not including here to avoid information redundancy)

//XM-like interchange format's effects
 #define XM_NOP_PACKSIGN 0x80
 #define XM_NOTE_PACKSIGN 0x81
 #define XM_INST_PACKSIGN 0x82
 #define XM_VOL_PACKSIGN 0x84
 #define XM_FX_PACKSIGN   0x88
 #define XM_FXVAL_PACKSIGN 0x90
 #define XM_GATEOFF_VALUE 97
 #define XM_PORTAMENTO_FX 3
 #define XM_VIBRATO_FX 4
 #define XM_NOTE_VOL_FX 0xC
 #define XM_MAIN_TEMPO_FX 0xF
 #define XM_MAIN_VOL_FX 0x10
 #define XM_LEGATO_FXVALUE 0xff //used together with portamento

//=================== Structure of the SWM module =============================

//HEADER - fixed size and positions described above
//-------------------------------------------------------
#define tuneheadersize 64 //$40 ;//don't modify (to keep compatibillity throughout versions)
//fix value positions in SWM module file (the commented are present in 'editor.inc'
//00..03: 'SWM1' identifier
#define FSPEEDPOS 4 //position of frame-speed (1..8)
//05: pattern-highlighting step-amount
#define SWM_HILI_POS 5 //used by converter 
//06: auto-advance-amount
#define SWM_AUTO_POS 6 //used by converter
//07: ON/OFF configuration bits (track-binding on/off, rasterbar-display on/off, default-followplay on/off, instrument auto-typing on/off, etc.)
#define SWM_CBIT_POS 7 //used by converter?
//08..0A: mute/solo switches
#define SWM_MUTE_POS 8 //used by converter
//0B: default pattern-length (for unspecified patterns
#define SWM_DEFP_POS 11 //used by converter
#define SEQAMOPOS 12 //$C  ;//position of Amount of sequences before empty-end (calculated by packer)
#define PTAMOUPOS 13 //$D  ;//position of Amount of patterns before empty-end - empty but lengthy patterns count too!!!
#define INSTAMPOS 14 //$E  ;//position of Amount of non-empty instruments before empty-end
#define CHRDLEPOS 15 //$F  ;//position of Length of packed chordtable
#define TMPLENPOS 16 //$10 ;//position of Length of packed tempoprogram-table
#define COLORTHEMEPOS 17 //$11 ;//position of colour-theme number
#define KEYBOARDTYPE_POS 18 //$12 ;//position of keyboard-type number (0-default, 4-GMC, $88-SDI)
#define DRIVERTYPE_POS 19   //$13 ;//position of player/driver-musicroutine type info (light/medium/full/extra), just an information, no restriction caused
//some bytes left for later expansions
#define AUTHORPOS 24 //(tuneheadersize-40) ;//$18

//MUSIC DATA itself:
//-------------------
//Sequences (that had pattern inside) - all expanded with its 1-byte size-info
//    like e.g.:  1,1,1,fe,(4), 2,2,2,ff,01,(5), 3,4,5,6,7,ff,02,(7), ...

//Patterns (that were referenced in sequences) - $ff endsignal changed to size-info, then expanded with length-info
//....like e.g.:  9,2,0,bd,1,9,2,(7),(6),  $7d,$22,2,2,0,(5),(5)  .....

//Instruments (that had something in their tables) - filtertable $ff signal changed to size of instrument without its name, and name is appended
//....like e.g.:  fix 16 bytes, then tables: 41,0,0,41,2,0,$ff, 88,00,00,$ff, $9F,03,01,($1e), 8char-NAME
//more precisely:
//instbase .BYTE %00011010 ;0 CONTROL REGISTER (HR TIMER,HRTYPE ,VIBRATO-TYPE,TIED PW/CTF)
//         .BYTE $0F,$F0  ;1 HR-ADSR
//         .BYTE $00,$F0  ;3 ADSR ON NOTESTART (GATE-ON)
#define SWM_AD_POS 3  //used for converter (Attack/Decay byte)
#define SWM_SR_POS 4  //used for converter (Sustair/Release byte)
//         .BYTE $00      ;5 VIBRATO (CALCULATED) FREQ+AMP (amplitude:high nibble)
#define SWM_INSVIBRATO_POS 5 //used for converter
//         .BYTE $00      ;6 VIBRATO DELAY OR AMPLITUDE-INCREMENT SPEED
#define SWM_VIBDELAY_POS 6 //used for converter
//         .BYTE $00      ;7 ARPEGGIO AND CHORD-SPEED (0=1X)
#define SWM_ARP_SPEED_POS 7 //used for converter
//         .BYTE $01      ;8 DEFAULT CHORD FOR INSTRUMENT
#define SWM_DEFCHORD_POS 8 //used for converter (and now SW too)
//         .BYTE $00      ;9 OCTAVE SHIFT (2's complement) - it's transpose essentially but in 12 steps by editor
#define SWM_OCTAVE_POS 9 //used for converter
//         .BYTE <(pulsprog - instbase) ;PWTBL1-INST1 ;A PW-TABLE POINTER RELATIVE TO INSTRUMENT-ADDRESS
#define SWM_PULSETBPT_POS 10 //$A used for converter
//         .BYTE <(filtprog - instbase) ;FLTBL1-INST1 ;B FILTER-TABLE POINTER RELATIVE TO INSTRUMENT-ADDRESS
//         .BYTE $00      ;C GATEOFF POINTER FOR WFARP-TABLE RELATIVE TO WF-TABLE POSITION
//         .BYTE $00      ;D GATEOFF POINTER FOR PW-TABLE RELATIVE TO PW-TABLE POSITION
//         .BYTE $00      ;E GATEOFF POINTER FOR FILTER-TABLE RELATIVE TO FILTER-TABLE POS.
//         .BYTE $00      ;F 1ST FRAME WAVEFORM
//         .BYTE $ff  ;WF-PROGAM ;hardwired position at $10 (relative to instrument-base)
//pulsprog .BYTE $Ff  ;PW-PROGRAM        
//filtprog .BYTE $Ff ;FILTER-PROGRAM
#define WFTABLEPOS 16 //$10 ;//WF-ARP TABLE BASE-POSITION RELATIVE TO INSTRUMENT-BASEPOSITION

//Chordtable- chords after each other, separated with $7e/$7f - the chordtable-size is given in CHRDLEPOS in the header. if 0, nothing is here
//....like e.g.:  0,3,7,7e, 0,4,7,A,$7f, ...

//Tempotable- tempo-programs separated by bytes with bit7=ON. tempotable-size is given in TMPLENPOS in header. if 0, nothing stored
//....like e.g.:  08,07,06,85, 20,20,10,90, ...

//Subtune-funktempos - left/rigth values in pair, based on number of subtunes (sequences/3) - one is always stored even if no sequence
//....like e.g.:  88,84, 08,84, 20,90, ... 
//-------------------------------------------------------------------------

//-----------------global variable declarations---------------------------------
char InputFileName[FILENAME_LENGTH_MAX]; //input-file described as 1st command-line parameter
char InputTypeID[TYPE_ID_LENGTH_MAX]; //type identifier string inside the source file 
char OutputFileName[FILENAME_LENGTH_MAX]; //output-file described as 2nd command-line parameter
FILE *InputFile, *OutputFile; //in/out tune-file descriptors
unsigned char TUNEDATA[TUNEDATA_SIZE_MAX]; //tune-file-data loaded in memory for conversion
unsigned int InputSize=0,OutputSize=0;  //detected size of the loaded/saved tune
unsigned int TYPE_ID_OFFSET,TYPE_ID_LENGTH; //offset & length in file where the file-type ID can be found
unsigned long MIDIdeltas[MIDI_CHANNELS_MAX][MIDI_TRACKLEN_MAX], ChannelDelta[MIDI_CHANNELS_MAX]; //midi delta-time-data for different channels
unsigned char TimeSignature[8]={4,2,18,8}; //set default MIDI time-signature
unsigned char MIDInotes[MIDI_CHANNELS_MAX][MIDI_TRACKLEN_MAX]; //midi note-data for different channels
unsigned char MIDIvelos[MIDI_CHANNELS_MAX][MIDI_TRACKLEN_MAX]; //midi velocity-data for different channels
unsigned int MIDItrkIndex[MIDI_CHANNELS_MAX], OKtrk[MIDI_CHANNELS_MAX], TuneBPM=120, MIDItempo=500000;
unsigned char Digits[MIDI_VLV_LEN_MAX]; //stores variable length value digits temporarily when processing MIDI
unsigned char ActualTrack; //temp for multitrack MIDI's currently processed track-number
unsigned char PATTERNLENGTHS[256]; //array of musical lengths of patterns
unsigned char TUNETEMPO[32][2]; //subtune-tempos
unsigned int ActSWMptn, ColumnCount, PackSignPos, ADSRpoint;
unsigned int XMsongLength=0;
unsigned int XMrestartPos=0;

//for MIDI-to-ORM conversion (XM-like interchange)
unsigned int ORMpatternCount[SID_CHNAMOUNT],XMpattPos; //,SWMseqLength[SID_CHNAMOUNT];
unsigned char XMPATTERNS[SID_CHNAMOUNT][XM_PATTERNS_MAX][256*XM_COL_PER_TRK]; //holds registered (non-duplicated) vertical unpacked XM patterns
unsigned char XMpattRef[SID_CHNAMOUNT][256]; //XM pattern-bunch references to the original SWM pattern-numbers
unsigned char ORMchPos[5]={0,0,1,1,2};

unsigned char XMorderlist[256]; //result pattern-numbers of 3-to-1 orderlist conversion / imported XM orderlist
unsigned char XMinstName[XM_INSTRUMENTS_MAX][XM_INSNAME_SIZE]; //XM instrument-names
 unsigned int XMinsAttack[XM_INSTRUMENTS_MAX], XMinsDecay[XM_INSTRUMENTS_MAX], XMinsRelease[XM_INSTRUMENTS_MAX];
 unsigned char XMinsSustain[XM_INSTRUMENTS_MAX]; 

//for ORM-to-SWM conversion (XM-like interchange) 
unsigned int InstAmount,PatternAmount,SequenceAmount,SubtuneAmount,ChordAmount,SWMseqLength[SID_CHNAMOUNT],SWMpatternCount;
unsigned char SEQUENCES[32*SID_CHNAMOUNT][256]; //array to hold orderlist (3 sequences)
unsigned char XM_Note,XM_Ins,XM_Vol,XM_Fx,XM_FxVal;
unsigned char PATTERNS[256][256]; //array to hold SWM pattern-data
unsigned char PATTERNSIZES[256]; //array of effective SWM pattern-sizes
unsigned char XMinsVibDelay[XM_INSTRUMENTS_MAX],XMinsVibAmp[XM_INSTRUMENTS_MAX],XMinsVibFreq[XM_INSTRUMENTS_MAX];
unsigned char ORMorderlist[SID_CHNAMOUNT][ORM_SEQUENCE_LENGTH];
unsigned char ORMpatterns[5][ORM_MAX_PATTERN_AMOUNT][ORM_PATTERN_LENGTH];
unsigned char ORMprogTable[3][0x100];
int chPattMin[SID_CHNAMOUNT]={0,0,0};
int PrevchPattMin=0;
unsigned char ORMheader[ORM_HEADER_SIZE];
unsigned char InstList[0x40];

unsigned char XMpattTemp[SID_CHNAMOUNT][256*XM_COL_PER_TRK]; //holds three temporarily read vertical unpacked XM patterns
 
char AUTHORINFO[256]; //array to hold tune author-info

//-----------------function-prototype declarations------------------------------
void WaitKeyPress(); void ProcessInput(); 
void ProcessMIDI(); void ConvertTune(); 
char* FilExt(char *filename); void ChangeExt(char *filename,char *newExt); void CutExt(char *filename);
void ProcessORM(); void ProcessMIDI(); void ProcessMIDItrack();
void GenerateORM(); void GenerateSWM();
unsigned int BEwordFromFile(); unsigned long ReadVarLenNum(); 
int LEwordToFile(unsigned int DataToWrite); unsigned int LEwordFromFile();

//==============================================================================

int main(int argc, char *argv[])
{
 printf ("\n====================================\n"
         "1raster-tracker music v1.x converter\n"
         "====================================\n");

 if ( (argc==2 && !strcmp(argv[1],"-help")) || argc==1 )
 {
  printf ("Converts MIDI to ORM, or ORM to SWM,\n"
          "and S00 to SID.                     \n"
          "Usage syntax is pretty simple:      \n"
          "ORMconvert <inputfile> [outputfile] \n"
          "------------------------------------\n"
          "MIDI should be 3 channels max in 4/4\n"
          "ORM tableFx won't be converted to SW\n"
          "(Formats will depend on extensions.)\n"
          "If you don't specify an output-file,\n"
          "name of the input file will be used \n"
          "as output-filename. Existing files  \n"
          "will be overwritten automatically.  \n"
          "------------------------------------\n"
          " 2013 Hermit Soft. (Mihaly Horvath) \n"
          "====================================\n");
  printf("\nPress any key...\n\n");
  getchar();
  exit(1);
 }
 else
 {
  InputFile = fopen(argv[1],"rb"); //!!!important "rb" is for binary (not to check text-control characters)
  if ( InputFile == 0 )
  {
   printf( "Could not open input-file...\n" );
   WaitKeyPress();exit(2);
  }
  else 
  {
   strcpy(InputFileName,argv[1]);
   if (argc==3) strcpy(OutputFileName,argv[2]);
   ProcessInput(); printf("\n\n**** Module processed. Going to Convert ****.\n---------------------------------------------\n");
   fclose(InputFile);
   ConvertTune();
  }
 }
  WaitKeyPress();
  return 0;
}


//==========================FUNCTIONS=====================================================
void ProcessInput()
{
 printf("Input-file name: %s \n",InputFileName);
 
 printf("Input-file extension: %s \n",FilExt(InputFileName));

 //read extension and decide if input is SWM or XM
 if ( !strcmp(FilExt(InputFileName),".ORM") || !strcmp(FilExt(InputFileName),".PRG") || !strcmp(FilExt(InputFileName),".orm") || !strcmp(FilExt(InputFileName),".prg") )
 {
  TYPE_ID_OFFSET=2; TYPE_ID_LENGTH=4;
  ProcessORM();
 }
 else if ( !strcmp(FilExt(InputFileName),".S00") ||  !strcmp(FilExt(InputFileName),".s00") )
 {
  printf("Converting .S00 format to .SID format.");
 }
 else if ( !strcmp(FilExt(InputFileName),".MID") || !strcmp(FilExt(InputFileName),".mid") )
 {
  TYPE_ID_OFFSET=MIDI_ID_OFFSET; TYPE_ID_LENGTH=MIDI_ID_SIZE;
  ProcessMIDI();
 } 
 else
 { 
  printf("\n*** Unsupported file-extension... ***\n");
  fclose(InputFile);WaitKeyPress();exit(3);
 }
}

//-----------------------------------------------------------------------------
void ConvertTune()  //convert to target format
{
 int Data;
 printf("Deciding on output-format and filename.\n");
 
  //read extension and decide if input is ORM or MIDI
 if (strlen(OutputFileName)==0) //if no output-file specified, give the same name and opposite/obvious extension
 { //no output-filename given
  printf("No output-file given. Using input-filename, appending extension.");
  strcpy(OutputFileName,InputFileName); 
  if ( !strcmp(FilExt(InputFileName),".ORM") || !strcmp(FilExt(InputFileName),".orm") )
  { //no output given
   ChangeExt(OutputFileName,".swm"); //append corresponding extension
   printf("\nConverting ORM "); GenerateSWM();
  }
  else if ( !strcmp(FilExt(InputFileName),".MID") || !strcmp(FilExt(InputFileName),".mid") )
  { //no output given
   ChangeExt(OutputFileName,".orm"); //append corresponding extension
   printf("\nConverting MIDI "); GenerateORM();
  }  
  else if ( !strcmp(FilExt(InputFileName),".S00") || !strcmp(FilExt(InputFileName),".s00") ) //if input-format is .S00, converting it to .sid
  { //no output given
   ChangeExt(OutputFileName,".sid"); //append with new extension
   printf("\n*** Converting .S00 VICE C64-file to .sid file... ***");    
   InputFile = fopen(InputFileName,"rb"); //!!!important "rb" is for binary (not to check text-control characters)
   OutputFile = fopen(OutputFileName,"wb"); //!!!important "wb" is for binary (not to check text-control characters)
   fseek(InputFile,VICE_P00_OFFSET,0); //go right after end of .S00 'C64File' header (omit header from .seq)
   while( (Data=fgetc(InputFile)) != EOF ) { fputc(Data,OutputFile); } //copy source to target
   fclose(InputFile); fclose(OutputFile);
   printf("\n*** Conversion done, file '%s' is generated... ***\n",OutputFileName);
  } 
  else
  { //no output given
   ChangeExt(OutputFileName,".swm"); //append new extension
   printf("\nConverting ORM "); GenerateSWM();
  }
 } 
 else if ( !strcmp(FilExt(OutputFileName),".swm") || !strcmp(FilExt(OutputFileName),".prg") )
 { //output name & SWM extension given
  if ( !strcmp(FilExt(InputFileName),".orm") )  { printf("\nConverting ORM "); GenerateSWM(); }
  else if ( !strcmp(FilExt(InputFileName),".SWM") || !strcmp(FilExt(InputFileName),".PRG") ||  !strcmp(FilExt(InputFileName),".swm") || !strcmp(FilExt(InputFileName),".prg") )  //what if input format is the same (SWM)?
  { printf("\n***Convert SWM to SWM? ***\n"); WaitKeyPress(); exit(111); }
 }
 else if ( !strcmp(FilExt(OutputFileName),".orm") )
 { //output name & ORM extension given
  if ( !strcmp(FilExt(InputFileName),".orm") || !strcmp(FilExt(InputFileName),".ORM") ) { printf("\n***Convert ORM to ORM? ***\n"); WaitKeyPress(); exit(111); } //what if input format is the same
  else { printf("\nConverting MIDI "); GenerateORM(); }
 }
 else //if output extension not found, automatic format (opposite of input-format)
 {
  if ( !strcmp(FilExt(InputFileName),".MID") || !strcmp(FilExt(InputFileName),".mid") )
  { //input-file is MIDI, no output-format/extension was given
   ChangeExt(OutputFileName,".orm"); //append with corresponding extension
   printf("\nConverting MIDI "); GenerateORM();
  }
  else if ( !strcmp(FilExt(InputFileName),".S00") || !strcmp(FilExt(InputFileName),".s00") ) //if input-format is .S00, converting it to .sid
  { //no output format/extension given
   ChangeExt(OutputFileName,".sid"); //append with new extension
   printf("\n*** Converting .S00 VICE C64-file to .sid file... ***");    
   InputFile = fopen(InputFileName,"rb"); //!!!important "rb" is for binary (not to check text-control characters)
   OutputFile = fopen(OutputFileName,"wb"); //!!!important "wb" is for binary (not to check text-control characters)
   fseek(InputFile,VICE_P00_OFFSET,0); //go right after end of .S00 'C64File' header (omit header from .seq)
   while( (Data=fgetc(InputFile)) != EOF ) { fputc(Data,OutputFile); } //copy source to target
   fclose(InputFile); fclose(OutputFile);
   printf("\n*** Conversion done, file '%s' is generated... ***\n",OutputFileName);
  } 
  else
  { //input-file is ORM or P00, no output-format was given
   ChangeExt(OutputFileName,".swm"); //append with new extension
   printf("\nConverting ORM "); GenerateSWM(); //preferred output-format is XM for SWM
  }
 }
} 


//===========================================================================================
void ProcessMIDI() //check MIDI type and version--------------------------------
{
 unsigned int PrevVelo,OKtrkAmount,MinimumDelta[SID_CHNAMOUNT],MaximumDelta[SID_CHNAMOUNT],i,j,k,l,BarPulses,NoteResolution; //counts bytes in loops
 unsigned int RowPulses, SourceIndex; 
 unsigned char CurrentNote, CurrentVelo, FirstRow, PtnMatch, PattTemp[256*XM_COL_PER_TRK]; 
 int CurrentDelta; //,DeltaDiff;
 float BarQuarters;

 printf("Processing MIDI file...\n");
 
 //display filesize and format
 fseek(InputFile, TYPE_ID_OFFSET, 0); fgets(InputTypeID, TYPE_ID_LENGTH+1, InputFile);
 fseek(InputFile, MIDI_VERSION_OFFSET, 0); InputMIDIver=BEwordFromFile();
 fseek(InputFile, 0, SEEK_END); InputSize=ftell(InputFile); fseek(InputFile, 0, SEEK_SET);
 printf("Input-file size: %d bytes \n",InputSize);
 printf("Input-filetype ID: \"%s\" (at offset %d) , version: \"\%X\" \n",InputTypeID,TYPE_ID_OFFSET,InputMIDIver);
 
 for(i=0;i<MIDI_CHANNELS_MAX;i++) MIDItrkIndex[i]=0; //init MIDI track/channel pointers before conversion
 
 if (!strcmp(InputTypeID,MIDI_ID))
 { //check module-version, if doesn't match, notify but try to do...  
  fseek(InputFile, MIDI_DIVISION_OFFSET, 0); MIDI_DIVISION=BEwordFromFile(); printf("Delta-timing resolution (units per quarter-note - PPQN): $%x ",MIDI_DIVISION);
  if (InputMIDIver==2) {printf("\nProcessing multi-song MIDI's 1st track...");InputMIDIver=0;} //only 1st tune processed of version 2
  if (InputMIDIver==0)
  {
   printf("\nProcessing single-track MIDI...");
   fseek(InputFile, (MIDI_TRACKS_OFFSET+4), 0); MIDI_TRACK_SIZE[0]=256*BEwordFromFile()+BEwordFromFile(); printf("\nSize of the track (containing all channels): $%x bytes\n",(unsigned int)MIDI_TRACK_SIZE[0]);
   ProcessMIDItrack(); //fill MIDI delta/note/velocity arrays of channels/tracks
  }
  else if (InputMIDIver==1)
  {
   printf("\nProcessing multi-track MIDI...");
   fseek(InputFile, MIDI_TRACK_AMOUNT_OFFSET, 0); MIDI_TRACK_AMOUNT=BEwordFromFile(); printf("\nAmount of MIDI-tracks (channels): $%x \n",MIDI_TRACK_AMOUNT);
   MIDI_TRACK_OFFSET[0]=MIDI_TRACKS_OFFSET; //init
   for(i=0;i<MIDI_TRACK_AMOUNT;i++) //positioning to beginning of tracks
   {
    fseek(InputFile, (MIDI_TRACK_OFFSET[i]+4), 0); MIDI_TRACK_SIZE[i]=256*BEwordFromFile()+BEwordFromFile(); 
    printf("\nProcessing Track $%x: offset $%x , size $%x",i+1,(unsigned int)MIDI_TRACK_OFFSET[i],(unsigned int)MIDI_TRACK_SIZE[i]);
    ActualTrack=i; ProcessMIDItrack(); //fill MIDI delta/note/velocity arrays of channels/tracks
    MIDI_TRACK_OFFSET[i+1]=MIDI_TRACK_OFFSET[i]+4+4+MIDI_TRACK_SIZE[i];
   }
  }
 }
 else
 {
  printf("\n*** The MIDI is of unknown format... ***\n");
  fclose(InputFile);WaitKeyPress();exit(4);
 }

 //tune processed, select only tracks which have note events
 printf("\nTune processing finished...");
 for(i=OKtrkAmount=0;i<MIDI_CHANNELS_MAX;i++)
 { 
  if(MIDItrkIndex[i])
  {
   printf("\nChannel %d: %d note events",i+1,MIDItrkIndex[i]); //for(j=0;j<MIDItrkIndex[i];j++) printf("%x*%x ",MIDIdeltas[i][j],MIDInotes[i][j]);
   OKtrk[OKtrkAmount]=i;OKtrkAmount++; //register non-empty tracks
  }
 }
 //note ON/OFF and velocity are in channels at array MIDIdeltas[], MIDInotes[], MIDIvelos[]

 //calculate minimal and maximal delta values - might be used to automatically determine needed note-resolution
 for(i=0;(i<SID_CHNAMOUNT)&&(i<OKtrkAmount);i++) //only care about 1st 3 channels wchich contain notes
 {
  MinimumDelta[i]=MaximumDelta[i]=MIDI_DIVISION; PrevVelo=0; //init
  for(j=0;j<MIDItrkIndex[OKtrk[i]];j++) 
  { //examine smallest delta (except 0 of course) to guess needed resolution
   if ( (MIDIdeltas[OKtrk[i]][j]) && (MIDIdeltas[OKtrk[i]][j]<MinimumDelta[i]) && (PrevVelo)) MinimumDelta[i]=MIDIdeltas[OKtrk[i]][j];
   if (MIDIdeltas[OKtrk[i]][j]>MaximumDelta[i]) MaximumDelta[i]=MIDIdeltas[OKtrk[i]][j];
   PrevVelo=MIDIvelos[OKtrk[i]][j];
  }
  printf("\nTrack $%x Min.delta: $%x (1/%d quarternote), Max.delta: $%x (%.1f quarternotes)",OKtrk[i]+1,MinimumDelta[i],MIDI_DIVISION/MinimumDelta[i],MaximumDelta[i],(float)MaximumDelta[i]/(float)MIDI_DIVISION);
 }

 //determine pattern-lengths for ORM
 BarQuarters=(float) (4.00/pow(2,(float)TimeSignature[1]))*TimeSignature[0];  //calculate how many quarter notes and deltas are in a bar (can be smaller than 1 if e.g. unusual 2/8 = 1/4 time signature)
 BarPulses = (unsigned int)(BarQuarters*MIDI_DIVISION); 
 NoteResolution=16; //16th notes resolution
 printf("\nThere are %.2f quarter notes (%.2f*$%x = $%x pulses) in a Bar.",BarQuarters,BarQuarters,MIDI_DIVISION,BarPulses);
 for(i=0;i<maxptnamount;i++) { PATTERNLENGTHS[i]=ORM_PATTERN_LENGTH; }  //(unsigned int)(BarQuarters*NoteResolution/2); } 
 RowPulses=MIDI_DIVISION*(4.00/NoteResolution);
 printf("\nOne pattern-row takes $%x pulses",RowPulses);
 printf("\nPattern-lengths will be $%x rows with %dth-note resolution = $%x pulses",PATTERNLENGTHS[0],NoteResolution,PATTERNLENGTHS[0]*RowPulses); 
 //convert BPM to C64 tempo info - BPM is quarter-notes/minute in 4/4, but e.g. eighth-notes/minute for 6/8 or 8/8, etc. 
 //unsigned int BeatTime=(unsigned int)( MIDItempo*(float)(4.00/pow(2,(float)TimeSignature[1])) ); //microseconds for a beat
 TuneBPM=(60000000/MIDItempo)*(float)(pow(2,(float)TimeSignature[1])/4.00); printf("\nBeats Per Minute: %d",TuneBPM); //real BPM based on denominator
 //unsigned int BarTime=BeatTime*BarQuarters; //time of a bar in microseconds (from quarternote-microseconds MIDItempo)
 unsigned int ORMrowTime=MIDItempo*(4.00/NoteResolution); //time of ORM pattern-row
 unsigned int PALframeTime=20000; //microseconds a PAL (50Hz) frame takes
 unsigned int PALframesPerRow=(ORMrowTime/PALframeTime); //how much PAL frames build up a pattern-row (C64 tune speed)
 //if (PALframesPerRow<3) PALframesPerRow=3; //limit to usable speed
 TUNETEMPO[0][0]=PALframesPerRow+0x80; printf("\nC64 tempo (based on BPM %d and Time-signature denominator): $%x\n",TuneBPM,PALframesPerRow);

//----create ORM patterns and orderlists--------------------------------------------------------------
 printf("\n\n*** Generating patterns from the MIDI-datastream. ***");
 XMsongLength=XMrestartPos=0; //init
 for(i=0; i<SID_CHNAMOUNT; i++) //only care about 1st 3 channels wchich contain notes
 {
  ORMpatternCount[i]=SourceIndex=k=0;   //init
  if (i<OKtrkAmount)
  {
   printf("\n\nGenerating patterns from track $%x: \n",OKtrk[i]+1);
   FirstRow=1; CurrentDelta=MIDIdeltas[OKtrk[i]][SourceIndex]; printf("\nDelta-time of 1st Note: $%x pulses\n",CurrentDelta); //init
   while ( ( SourceIndex < MIDItrkIndex[OKtrk[i]] ) && ( (ORMpatternCount[i]+1)<maxptnamount ) && (k<seqlength) )
   {
    printf("pattern $%x, ",ORMpatternCount[i]);  //  printf("\n");
    for (j=0;j<PATTERNLENGTHS[1]*XM_COL_PER_TRK;j++) PattTemp[j]=0; //init pattern
    for(j=0;j<PATTERNLENGTHS[1];j++) //j counts pattern-rows
    {
     if ( SourceIndex >= MIDItrkIndex[OKtrk[i]] ) break; //don't continue pattern-creation if MIDI data is over
     for(l=0;l<RowPulses;l++) //counts pattern-row pulses one-by-one, quantizes in 50% range back&forth
     {
      if (CurrentDelta==0) //delay expired, put event into this or next row, depending on 50% treshold
      { //if actual note timed out, register it and go to next in MIDI-track
       do  //wait in place if zero-delay(s) happened - hopefully won't cause endless loops
       {
        CurrentNote=MIDInotes[OKtrk[i]][SourceIndex]; CurrentVelo=MIDIvelos[OKtrk[i]][SourceIndex];
        if ( CurrentNote<12 || CurrentNote>=106) CurrentNote=(12-1); //limit note-range to ORM
        if ( l < (RowPulses/2) ) //check if less than 50%
        { //if smaller than 50%, put in this row...
         if ( (PattTemp[j*XM_COL_PER_TRK]==0) || (PattTemp[j*XM_COL_PER_TRK]==XM_GATEOFF_VALUE) ) //touch data only if there's no note data in the place already (one-shot writing)
         { PattTemp[j*XM_COL_PER_TRK]=(CurrentVelo) ? CurrentNote-(12-1) : XM_GATEOFF_VALUE ;} 
        }
        else 
        {  //if bigger than 50%, put to next row instead
         if ( (PattTemp[(j+1)*XM_COL_PER_TRK]==0) || (PattTemp[(j+1)*XM_COL_PER_TRK]==XM_GATEOFF_VALUE) ) //touch data only if there's no note in the place already (one-shot writing)
         { PattTemp[(j+1)*XM_COL_PER_TRK]=(CurrentVelo) ? CurrentNote-(12-1) : XM_GATEOFF_VALUE ; }
        }
        SourceIndex++; CurrentDelta=MIDIdeltas[OKtrk[i]][SourceIndex];
       } 
       while (CurrentDelta==0); //wait in place if zero-delay(s) happened 
       //printf("%x*",CurrentDelta);
      }
      CurrentDelta--; //roll through 0 delays, maximum Rowpulses/2 zero-delays allowed
     }
     //printf("$%2X ",PattTemp[j*XM_COL_PER_TRK]);
    }
   
    //compare the resulted vertical XM pattern to existing ones, register if not found, reuse existing if found
    //(might be good idea to look for transposed patterns here or later, were relative note-intervals and effects match)
    PtnMatch=0; //init
    for (l=1;l<=ORMpatternCount[i];l++) //check registered XM patterns from beginning
    {
     for (j=0;j<(PATTERNLENGTHS[1]*XM_COL_PER_TRK);j++) //compare bytes one-by-one to pattern 'l'
     {
      if (PattTemp[j]!=XMPATTERNS[i][l-1][j]) {PtnMatch=0;break;} //mismatch found
      else PtnMatch++; //count matching bytes
     }
     if (PtnMatch) break;
    }
    //register or reuse patterns based on PtnMatch
    if (PtnMatch) //if match is found, reusing pattern 'l'
    {
     printf("$%2x  ",l-1);
     XMpattRef[i][k]=l-1;
    }
    else //no match found, registering new pattern
    {
     XMpattRef[i][k]=ORMpatternCount[i]++;
     for (j=0;j<PATTERNLENGTHS[1]*XM_COL_PER_TRK;j++) //copy the registered temporary vertical XM pattern to pattern-registry 
     { 
      XMPATTERNS[i][ORMpatternCount[i]-1][j]=PattTemp[j]; //printf("%2X,",XMPATTERNS[ORMpatternCount][j]);
     }
     printf("$%2x  ",XMpattRef[i][k]);
    }
    k++;
    if (i==0) { XMorderlist[k]=k; XMsongLength=k; }
   }
  }
  else
  { //handle empty sequences possibly left
   ORMpatternCount[i]++;
   printf("\n\nGenerating empty sequence with empty pattern $%x... \n",ORMpatternCount[i]);
   for (k=0;k<XMsongLength;k++) 
   {
    XMpattRef[i][k]=ORMpatternCount[i]; //last unused pattern empty pattern
   }
  }
 }
 
}

//------------------------------------------------------------------------
void ProcessMIDItrack()
{
 unsigned char EventTemp,Event,DestinationTrack,MetaEvent,MIDIdata1,MIDIdata2,MIDInote,MIDIvelo;
 unsigned int i,j; //set default BPM
 unsigned long MIDIdelta,MetaDataLength; //set default tempo (120bpm in 4/4 mode)

 for(i=0;i<MIDI_CHANNELS_MAX;i++) ChannelDelta[i]=0; //init

 //file pointer points to beginning of track here - read and put MIDI notes to corresponding arrays of channels
 while (MetaEvent!=0x2F) //for(i=0;i<MIDI_TRACK_SIZE[0];)
 {
  MIDIdata1=MIDIdata2=0; //init
  MIDIdelta=ReadVarLenNum(); //i++; //printf ("$%x ",MIDIdelta);
  EventTemp=fgetc(InputFile); //examine Event
  if (EventTemp&0x80) Event=EventTemp; //if legal event 'status' byte
  else fseek(InputFile, -1, SEEK_CUR); //otherwise if no status-byte given, we use the previously read 'Event'
  if (InputMIDIver!=1) //if version 0 (single-track) or 2, distribute based on channel-number
  {  
   DestinationTrack=Event&0xF; 
   for(i=0;i<MIDI_CHANNELS_MAX;i++) ChannelDelta[i]+=MIDIdelta;   //accumulate timing for not touched tracks
  }
  else 
  {
   DestinationTrack=ActualTrack; //if version 1, distribute channel notes based on track-number
   ChannelDelta[ActualTrack]+=MIDIdelta; 
  }

  if(Event==0xFF) //check if Meta-event
  {
   MetaEvent=fgetc(InputFile); MetaDataLength=ReadVarLenNum();
   switch (MetaEvent)
   {
    case 0x01 : //text event
    {
	 printf("\nText: ");
	 for(j=0;j<MetaDataLength;j++) printf("%c",fgetc(InputFile));
	 break;
    }
    case 0x02 : //copyright note
    {
	 printf("\nCopyright text: ");
	 for(j=0;j<MetaDataLength;j++) 
     {
      MIDIdata1=fgetc(InputFile);
      if (j<40)  AUTHORINFO[j]=MIDIdata1;
      printf("%c",MIDIdata1);
     }
	 break;
    }
    case 0x03 : //sequence/track name
    {
	 printf("\nTrack-name: ");
	 for(j=0;j<MetaDataLength;j++) printf("%c",fgetc(InputFile));
	 break;
    }
    case 0x2F : //end of track
    {
	 break;
    }
    case 0x51 : //tempo setting
    {
	 printf("\nTune-tempo (microseconds per quarter-note): ");
	 MIDItempo=0; //reset value, 24bit value (3 bytes)
	 for(j=0;j<MetaDataLength;j++) MIDItempo+=pow(256.00,(float)(2-j))*fgetc(InputFile); //printf("%x,",fgetc(InputFile));
     printf("%d  -> %d quarter-notes/minute \n",(unsigned int)MIDItempo,60000000/MIDItempo); 
	 break;
    }
    case 0x58 : //Time signature
    {
	 printf("\nTime-signature: ");
	 for(j=0;j<MetaDataLength;j++) TimeSignature[j]=fgetc(InputFile); //printf("%x ",fgetc(InputFile));
	 printf("Numerator: %d ; Denominator(2:quarter-note,3:eigth,etc.): %d",TimeSignature[0],TimeSignature[1]);
     printf("\n                MIDI-clocks per metronome-tick: %d \n                Notated 32nd notes in a quarter-note: %d \n",TimeSignature[2],TimeSignature[3]);
	 break;
    }
    default : 
    {
	 for(j=0;j<MetaDataLength;j++) fgetc(InputFile); //printf("%x ",fgetc(InputFile));
	 break;
	}
   }
  }
  else if (Event==0xF0) //check if System Exclusive Event
  {
   while (fgetc(InputFile)!=0xF7) ; //simply roll through sysex till its end
  }
  else if (Event==0xF7) 
  {
   while (fgetc(InputFile)!=0xF7) ; //simply roll through sysex till its end
  }
  else //normal MIDI events
  {
   switch (Event&0xF0)
   {
	case 0x90 : //Note-On MIDI event
	{ //put different channels' notes in separate array-tables
	 MIDIdeltas[DestinationTrack][MIDItrkIndex[DestinationTrack]]=ChannelDelta[DestinationTrack]; 
     //printf("*%x* ",ChannelDelta[DestinationTrack]);
     ChannelDelta[DestinationTrack]=0;
	 MIDIdata1=MIDInote=fgetc(InputFile); //printf("$%x*%x-%x-",(unsigned int)MIDIdelta,Event,MIDInote);
	 MIDIdata2=MIDIvelo=fgetc(InputFile); //printf("%x ",MIDIvelo);
	 MIDIvelos[DestinationTrack][MIDItrkIndex[DestinationTrack]]=MIDIvelo; 
	 MIDInotes[DestinationTrack][MIDItrkIndex[DestinationTrack]]=MIDInote; //(MIDIvelo==0) ? GATEOFFX : MIDInote;
	 MIDItrkIndex[DestinationTrack]++;
	 break;
    }
    case 0x80 : //Note-OFF MIDI event
    {
	 MIDIdeltas[DestinationTrack][MIDItrkIndex[DestinationTrack]]=ChannelDelta[DestinationTrack]; ChannelDelta[DestinationTrack]=0;
     MIDIdata1=MIDInote=fgetc(InputFile); //printf("%x-%x-%x-",Event&0xF,(unsigned int)MIDIdelta,MIDInote);
	 MIDInotes[DestinationTrack][MIDItrkIndex[DestinationTrack]]=MIDIdata1; //GATEOFFX;
	 MIDIdata2=MIDIvelo=fgetc(InputFile); //printf("%x ",MIDIvelo);
	 MIDIvelos[DestinationTrack][MIDItrkIndex[DestinationTrack]]=0; //MIDIvelo;
	 MIDItrkIndex[DestinationTrack]++;
	 break;
    }
    case 0xC0 : {MIDIdata1=fgetc(InputFile);break;}
    case 0xD0 : {MIDIdata1=fgetc(InputFile);break;}
	default :
	{
	 MIDIdata1=fgetc(InputFile); MIDIdata2=fgetc(InputFile);
    }
   }
   //printf("*%x-%x-%x ",Event,MIDIdata1,MIDIdata2);
  }
 }
}

unsigned char FreqTablePalLo[]={ //  C   C#    D   D#    E    F   F#    G   G#    A   A#   B
	                             0, 0x17,0x27,0x39,0x4b,0x5f,0x74,0x8a,0xa1,0xba,0xd4,0xf0,0x0e,  // 1
                                    0x2d,0x4e,0x71,0x96,0xbe,0xe8,0x14,0x43,0x74,0xa9,0xe1,0x1c,  // 2
                                    0x5a,0x9c,0xe2,0x2d,0x7c,0xcf,0x28,0x85,0xe8,0x52,0xc1,0x37,  // 3
                                    0xb4,0x39,0xc5,0x5a,0xf7,0x9e,0x4f,0x0a,0xd1,0xa3,0x82,0x6e,  // 4
                                    0x68,0x71,0x8a,0xb3,0xee,0x3c,0x9e,0x15,0xa2,0x46,0x04,0xdc,  // 5
                                    0xd0,0xe2,0x14,0x67,0xdd,0x79,0x3c,0x29,0x44,0x8d,0x08,0xb8,  // 6
                                    0xa1,0xc5,0x28,0xcd,0xba,0xf1,0x78,0x53,0x87,0x1a,0x10,0x71,  // 7
                                    0x42,0x89,0x4f,0x9b,0x74,0xe2,0xf0,0xa6,0x0e,0x33,0x20,0      /* 8 */ };

unsigned char FreqTablePalHi[]={ //  C   C#    D   D#    E   F    F#    G   G#   A    A#   B
		                         0, 0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x02,  // 1
                                    0x02,0x02,0x02,0x02,0x02,0x02,0x03,0x03,0x03,0x03,0x03,0x04,  // 2
                                    0x04,0x04,0x04,0x05,0x05,0x05,0x06,0x06,0x06,0x07,0x07,0x08,  // 3
                                    0x08,0x09,0x09,0x0a,0x0a,0x0b,0x0c,0x0d,0x0d,0x0e,0x0f,0x10,  // 4
                                    0x11,0x12,0x13,0x14,0x15,0x17,0x18,0x1a,0x1b,0x1d,0x1f,0x20,  // 5
                                    0x22,0x24,0x27,0x29,0x2b,0x2e,0x31,0x34,0x37,0x3a,0x3e,0x41,  // 6
                                    0x45,0x49,0x4e,0x52,0x57,0x5c,0x62,0x68,0x6e,0x75,0x7c,0x83,  // 7
                                    0x8b,0x93,0x9c,0xa5,0xaf,0xb9,0xc4,0xd0,0xdd,0xea,0xf8,0      /* 8 */ };
 

//===========================================================================================
void GenerateORM() //---- Export ORM data from the imported XM/MIDI/SNG/etc. data -----------
{
 // these global variables/arrays should be loaded with imported music-data:
 //  ORMpatternCount - number of imported patterns
 //  (ChordAmount - number of possibly imported chords)
 //  (SequenceAmount - number of sequences - now hardwired to SID_CHNAMOUNT=3 below)
 //  InstAmount - amount of imported instruments
 //  AUTHORINFO[] - assembled author-text, first $28 characters will be used for ORM output...
 //  XMsongLength - orderlist-length (in case of Goattracker SNG it should be a 3-element array instead)
 //  XMpattRef[] - 
 //  XMorderlist[] - converts to SEQUENCES[i][j] and ORMseqLength[i] then TUNEDATA[];
 //  XMPATTERNS[i][j+0..4] - pattern-arrays with these columns of j: note,instrument,volume,fx,fxvalue
 //  etc...                - converted to PATTERNS[i][ORMpattIndex], PATTERNLENGTHS[],PATTERNSIZES[], then TUNEDATA[]
 //  conclusion: NOT SUITABLE TO PRECISELY CONVERT FROM GOATTRACKER .SNG FORMAT. USING CONRAD'S CODE INSTEAD.
 //

 unsigned int h,i,j,DataIndex=0,ORMpattIndex;
 unsigned char ORM_Note,ORM_Ins,ORM_Fx,ORM_FxVal,PrevInst=0,PrevFx=0,PrevTempo=0,PrevVib=0,NOPcount=0,SeqData,PattData,PrevData;
 
 printf("to ORM (1raste-tracker v1.0) format. \n");
 printf("Output file-name will be: %s\n",OutputFileName); 
 
 //---------- assembling TUNEDATA array before writing to file ---------
 OutputSize=tuneheadersize; for(i=0;i<TUNEDATA_SIZE_MAX;TUNEDATA[i]=0,i++); //init
 ChordAmount=0; //might be used later

 //write out TUNE DATA to disk
 if ( !strcmp(FilExt(OutputFileName),".swm") ) strcat(OutputFileName,".prg"); //if only .swm given, append .prg
 OutputFile = fopen(OutputFileName,"wb"); // !!!important "wb" is for binary (not to check text-control characters)
 
 //write tune-header
 LEwordToFile(0x1000); //-2. load-address of PRG format
 for(i=0;i<0x100;i++) fputc(0x60,OutputFile); //'RTS' player-code area, not needed to contain code as 1raster-tracker loads music data only
 fputc(0,OutputFile); //pattern-start offset
 fputc((TUNETEMPO[0][0]-2)&0x7F,OutputFile); //tempo
 fwrite(ORM_HEADER_SAMPLE,(ORM_HEADER_SIZE-2-ORM_AUTHORINFO_SIZE),1,OutputFile); //sample ORM header
 fputs(ORM_AUTHORINFO_SAMPLE,OutputFile); //sample ORM title
 
 //write ORM sequences
 printf("\nConverting processed Orderlist to 3 ORM sequences"); 
 for(i=0;i<SID_CHNAMOUNT;i++) 
 {
  printf("\nSWM Sequence $%x: ",i);
  for (j=0;j<ORM_SEQUENCE_LENGTH;j++)
  {
   SeqData=XMpattRef[i][XMorderlist[j]];
   if (j<XMsongLength && (SeqData) && (SeqData<=ORM_AUTHORINFO_SIZE) ) ; else SeqData=0; //don't allow illegal patterns
   fputc(SeqData*ORM_PATTERN_LENGTH,OutputFile); printf("%2x ",SeqData);
  }
 }
 
 //write ORM patterns - XMPATTERNS[][] filled with needed XM pattern-data ---------------------
 //XMpattref[][] contains which pattern of XMPATTERNS array corresponds which SWM pattern 
 //XMpatternCount holds how much patterns are needed
 //Now, convert XM patterns to SWM equivalents 
 unsigned char XM_Note,Kuss=0,ORMpatt[ORM_PATTERN_LENGTH];
 unsigned char ptnL[256],ptnH[256];
 for (h=0; h<5; h++)
 {
  for(i=0;i<ORM_MAX_PATTERN_AMOUNT;i++) 
  {
   ORM_Note=0; printf("\nPattern $%2x: ",i);
   for(j=0; j<(ORM_PATTERN_LENGTH*XM_COL_PER_TRK); j+=XM_COL_PER_TRK) // j counts pattern-rows
   {
    XM_Note = XMPATTERNS[ORMchPos[h]][i][j+0];
    if (XM_Note)
    {
     if (XM_Note==XM_GATEOFF_VALUE) Kuss=1; else Kuss=0;
     switch (h)
     {
      case 0: case 2: ORM_Note=(Kuss)?0:FreqTablePalLo[XM_Note]; break; //channel 1 patterns
      case 1: case 3: ORM_Note=(Kuss)?0:FreqTablePalHi[XM_Note]; break; //channel 2 patterns
      case 4: ORM_Note = (Kuss)? (XM_Note==XM_GATEOFF_VALUE)?0x20:0 : XM_Note*2+0x20; break; //channel 3 patterns
     }
    }
    else if (Kuss) ORM_Note=0;
    ORMpatt[ORM_PATTERN_LENGTH-1-j/XM_COL_PER_TRK]=ORM_Note;
    printf("%2X ",ORM_Note);
   }
   fwrite(ORMpatt,ORM_PATTERN_LENGTH,1,OutputFile);
  }
 }
 
 //write program-table (empty of course)
 for(i=0;i<0x100;i++) { if (i>=0x20 && i<0xE0 && !(i%2)) fputc(0x21,OutputFile); else fputc(0,OutputFile); } //table-column 1
 for(i=0;i<0x100;i++)
 {
  if (i>=0x20 && i<0xE0) { if (!(i%2)) fputc(FreqTablePalLo[(i-0x20)/2],OutputFile); else fputc(i-1,OutputFile); } //table-column 2
  else fputc(i,OutputFile);
 }
 for(i=0;i<0xE8;i++) { if (i>=0x20 && i<0xE0 && !(i%2)) fputc(FreqTablePalHi[(i-0x20)/2],OutputFile); else fputc(0xFF,OutputFile); } //table-column 3 - jump-commands
   
 //fwrite(TUNEDATA,OutputSize,sizeof(unsigned char),OutputFile); 
 printf("\n\n**** Conversion is complete. **** \nSWM file '%s' generated... :)",OutputFileName);
 fclose(OutputFile);

 printf("\n-----------------------------------------------------\n");
}



//==========================================================================================
void ProcessORM() //--- import ORM data ---
{
  int h,i,j,k,l,ReadPitch,PtNumTemp;
  unsigned int PtnMatch,kNote;
  
  printf("Processing 1raster-tarcker ORM format...\n");
  
  fseek(InputFile, ORM_HEADER_OFFSET,0);
  for(i=0;i<ORM_HEADER_SIZE;i++) ORMheader[i]=fgetc(InputFile);
  
  fseek(InputFile, ORM_HEADER_OFFSET+ORM_AUTHORINFO_OFFSET, 0); 
  printf("\n");for(i=0;i<ORM_AUTHORINFO_SIZE;i++) {AUTHORINFO[i]=fgetc(InputFile); printf("%c",AUTHORINFO[i]);}
  fseek(InputFile, ORM_HEADER_OFFSET+ORM_TEMPO_OFFSET, 0); TUNETEMPO[0][0]=(LEwordFromFile()+2)|0x80; printf("\nDefault tune-tempo: $%X \n",TUNETEMPO[0][0]&0x7F);
  
 //process ORM orderlist----------------------------------------
 printf("\nProcessing ORM pattern-orderlist... \n");
 fseek(InputFile, ORM_ORDERLIST_OFFSET, 0); //position to beginning of Orderlist
 chPattMin[0]=chPattMin[1]=chPattMin[2]=PrevchPattMin=0; //init
 for(h=0;h<SID_CHNAMOUNT;h++)
 {
  chPattMin[h]=PrevchPattMin+SWMpatternCount; 
  SWMpatternCount=0; //init
  printf("\nChannel %2d: ",h);
  for(i=0;i<ORM_SEQUENCE_LENGTH;i++) 
  {
   ORMorderlist[h][i]=fgetc(InputFile); printf("$%.2X ",ORMorderlist[h][i]);
   if (ORMorderlist[h][i]/ORM_PATTERN_LENGTH>SWMpatternCount) SWMpatternCount++;
  }
  SWMpatternCount++;
  PrevchPattMin=chPattMin[h];
 }
 SWMpatternCount=chPattMin[h]+SWMpatternCount; //init

 //process ORM patterns-----------------------------------------------
 printf("\nProcessing ORM patterndata... \n");

 for (h=0;h<5;h++) //read all pattern-data to ORM pattern-array
 {
  printf("\nBlock $%2X",h);
  for (i=0;i<ORM_MAX_PATTERN_AMOUNT;i++)
  {
   printf("\n Pattern $%2X: ",i);   
   for(j=0;j<ORM_PATTERN_LENGTH;j++)
   {
    ORMpatterns[h][i][ORM_PATTERN_LENGTH-1-j]=fgetc(InputFile); 
    printf("%.2X ",ORMpatterns[h][i][ORM_PATTERN_LENGTH-1-j]);
   }
  }
 }
 
 //process ORM program-table--------------------------------------------
 printf("\n\nProcessing ORM Waveform-Pitch program-table...");
 
 for(h=0;h<3;h++)
 {
  //printf("\n Column %.2X:  ",h);
  for(i=0;i<0x100;i++)
  {
   ORMprogTable[h][i]=fgetc(InputFile); //printf("%.2X ",ORMprogTable[h][i]);
  }
 }

 unsigned char ValidInstr;
 printf("\n\nGenerating instrument table-index list based on pattern3...");
 InstAmount=2; //instruments reserved for the 1st and 2nd channel
 for(i=0;i<ORM_MAX_PATTERN_AMOUNT;i++)
 {
  for (j=0;j<ORM_PATTERN_LENGTH;j++)
  {
   for (k=2;k<=InstAmount; k++) //check if found new instrument-entry
   {
    if (ORMpatterns[4][i][j]==0 || ORMpatterns[4][i][j]==InstList[k]) break; 
   }
   for (l=ORMpatterns[4][i][j] ;l<ORMpatterns[4][i][j]+0x1E; l++) //check if program contains wf-arp.program at all. (Valid.) 
   {
	ValidInstr=0;
    if(ORMprogTable[0][l]!=0) {ValidInstr=1; break;}
    if(ORMprogTable[0][l]==0 && ORMprogTable[2][l]==0xFF) {ValidInstr=0;break;}
   }
   if (k==InstAmount+1 && ValidInstr==1) { InstList[InstAmount]=ORMpatterns[4][i][j]; InstAmount++;}
  }
 }
 printf("\nThere are %d unique instrument-entries in table: \n",InstAmount-2);
 for(i=2;i<InstAmount;i++) printf("Inst%.2X:%.2X  ",i,InstList[i]);
 
 
//turn ORM patterns to XM-like uncompressed patterns
 for (i=0;i<ORM_MAX_PATTERN_AMOUNT*SID_CHNAMOUNT;i++) PATTERNLENGTHS[i]=ORM_PATTERN_LENGTH;

 for (h=0;h<SID_CHNAMOUNT;h++) //1st 2 channels
 {
  ORMpatternCount[h]=0; //printf("\n"); //init
  for(i=0;i<ORM_MAX_PATTERN_AMOUNT;i++) //i counts patterns
  {
   //printf("\n ");
   for (j=0;j<ORM_PATTERN_LENGTH;j++)
   {
	XMpattPos=j*XM_COL_PER_TRK;
	if(h<2) 
	{ //channel 1 & 2
	 if (j==0) XMPATTERNS[0][i+chPattMin[h]+1][XMpattPos+1]=h+1; //put instruments for channel 1 & 2
	 ReadPitch=ORMpatterns[h*2+1][i][j]*0x100+ORMpatterns[h*2][i][j]; //printf("%.4X ",ReadPitch); 
	 for (k=0;k<=255;k++) //detect note from 16 bit pitch-data
	 {
	  if (FreqTablePalHi[k]*0x100+FreqTablePalLo[k]>=ReadPitch)
	  {
       XMPATTERNS[0][i+chPattMin[h]+1][XMpattPos+0]=k; //printf("%.2X-",k); //read note
       break;
      }
     }
    }
    else
    { //channel 3  
     if (ORMpatterns[4][i][j]) 
     { 
      for(k=2;k<=InstAmount;k++) 
      { 
	   if(InstList[k]==ORMpatterns[4][i][j]) break; 
	  }
	  if (k<InstAmount) {k++; kNote=1+12*4;}       
	  else {k=0;kNote=0;}
     }
     else {k=0;kNote=0;}
     XMPATTERNS[0][i+chPattMin[h]+1][XMpattPos+0]=kNote; //note
     XMPATTERNS[0][i+chPattMin[h]+1][XMpattPos+1]=k; //printf("%.2X-",k); //instrument-number
    }
   }
  }
 }
 
}


//===========================================================================================
void GenerateSWM() //---- Export SWM data from the imported data -----------
{
 printf("to SWM (SID-WIZARD v1.0) format. \n");
 printf("Output file-name will be: %s\n",OutputFileName); 
 
 /* these global variables/arrays should be loaded with imported music-data:
 //  SWMpatternCount - number of imported patterns
 //  InstAmount - amount of imported instruments
 //  AUTHORINFO[] - assembled author-text, first $28 characters will be used for SWM output...
 //  ORMorderlist[] - converts to SEQUENCES[i][j] and SWMseqLength[i] then TUNEDATA[];
 //  XMPATTERNS[i][j+0..4] - pattern-arrays with these columns of j: note,instrument,volume,fx,fxvalue
 //  etc...                - converted to PATTERNS[i][SWMpattIndex], PATTERNLENGTHS[],PATTERNSIZES[], then TUNEDATA[]
 */

 unsigned int i,j,k,l,DataIndex=0,SWMpattIndex;
 unsigned char SWM_Note,SWM_Ins,SWM_Fx,SWM_FxVal,PrevNote=0,PrevInst=0,PrevFx=0,PrevTempo=0,PrevVib=0,NOPcount=0,PattData,PrevData;
 
 printf("to SWM (SID-WIZARD v1.0) format. \n");
 printf("Output file-name will be: %s\n",OutputFileName); 
 
 //---------- assembling TUNEDATA array before writing to file ---------
 OutputSize=tuneheadersize; for(i=0;i<TUNEDATA_SIZE_MAX;TUNEDATA[i]=0,i++); //init
 SequenceAmount=SID_CHNAMOUNT; //only 3, because only 1 subtune gets generated
 PatternAmount=(SWMpatternCount<maxptnamount) ? SWMpatternCount : maxptnamount;
 ChordAmount=0; //might be used later

 //write SWM header
 for (i=0;SWM_ID[i];i++) TUNEDATA[i]=SWM_ID[i];  //0-3. SWM-ID
 TUNEDATA[FSPEEDPOS]=1;    //4.preferred framespeed
 TUNEDATA[SWM_HILI_POS]=4; //5.preferred step-highlighting amount - should depend on patternlength
 TUNEDATA[SWM_AUTO_POS]=1; //6.preferred auto-advance amount
 TUNEDATA[SWM_CBIT_POS]=1; //7.track-binding ON  (, other 'confbit' newsettings are 'OFF')
 for(i=0;i<SID_CHNAMOUNT;i++) TUNEDATA[SWM_MUTE_POS+i]=0xFF; //8-$A.solo all tracks
 TUNEDATA[SWM_DEFP_POS]=DEFAULTPTNLEN; //$B. default pattern-length - should depend on ORM patternlengths
 TUNEDATA[SEQAMOPOS]=SequenceAmount; //$C. sequence-amount
 TUNEDATA[PTAMOUPOS]=PatternAmount; //$D. pattern-amount calculated in pattern-converter
 TUNEDATA[INSTAMPOS]=InstAmount;    //$E. number of instruments 
 TUNEDATA[CHRDLEPOS]=ChordAmount;   //$F. chordtable-length (calculated in converter)
 TUNEDATA[TMPLENPOS]=0; //tempo-programs are not supported in XM/MIDI, leaving it out
 TUNEDATA[COLORTHEMEPOS]=0; //default colour-theme
 TUNEDATA[KEYBOARDTYPE_POS]=0; //default (GOATTRACKER-like) 2-row keyboard mode
 TUNEDATA[DRIVERTYPE_POS]=0; //normal music-player/driver routine version
  //....some bytes not used yet in SWM version 1. (kept for possible upcoming expansion)
 for(i=0;i<(tuneheadersize-AUTHORPOS);i++) TUNEDATA[AUTHORPOS+i]=toupper(AUTHORINFO[i]); //$18..$3f - author-info
 
 DataIndex=tuneheadersize; //init pack-target-address
 
 //write SWM sequences XMorderlist[] contains XM-orderlist, XMsongLength is its length
 printf("\nConverting processed ORM Orderlist to 3 SWM sequences"); 
 for(i=0;i<SequenceAmount;i++) 
 {
  printf("\nSWM Sequence $%x: ",i);
  for (j=0;j<ORM_SEQUENCE_LENGTH;j++)
  {
   SEQUENCES[i][j]=ORMorderlist[i][j]/ORM_PATTERN_LENGTH+chPattMin[i]+1; 
   printf("%2x ",SEQUENCES[i][j]);
  }
  SEQUENCES[i][j]=0xFF;j++; SEQUENCES[i][j]=XMrestartPos; j++; //put 'jump' delimiter and corresponding loop-address (will be harder with transpose)
  SWMseqLength[i]=j;  //if transpose gets supported, the SWM sequence-lengths will differ
 }
 for(i=0;i<SequenceAmount;i++)  //write orderlist to TUNEDATA-array
 {
  for (j=0;j<SWMseqLength[i];j++)
  {
   TUNEDATA[DataIndex]=SEQUENCES[i][j]; //copy SWM sequence to TUNEDATA[]
   DataIndex++;
  }
  TUNEDATA[DataIndex]=SWMseqLength[i]; DataIndex++; //write size of sequence into TUNEDATA[]
 }
 
 //write SWM patterns - XMPATTERNS[][] filled with needed XM pattern-data ---------------------
 //XMpatternCount holds how much patterns are needed
 //Now, convert XM patterns to SWM equivalents (make packed NOPs too)     // !!! SWM patterns start from 1
 printf("\n\nConverting processed patterndata to SWM format:  "); 
 for (i=1;i<=PatternAmount;i++) // i counts patterns
 {
  SWMpattIndex=PrevInst=PrevNote=PrevVib=0; //init non-linear SWM data pointer for SWM target and previous value buffers
  for(j=0; j<(PATTERNLENGTHS[i]*XM_COL_PER_TRK); j+=XM_COL_PER_TRK) // j counts pattern-rows
  {
   if (SWMpattIndex>=maxptnlen) { PATTERNLENGTHS[i]=(j/XM_COL_PER_TRK); break; } //don't allow bigger than allowed length of patterns
   XM_Note = XMPATTERNS[0][i][j+0];
   XM_Ins  = XMPATTERNS[0][i][j+1]; 
   //convert XM Ins., Vol/Fx to SWM ins., small-/big-FX --- mark every NOP as 0! -----------
   //CASE1 - if there's something to put into note-column (note / note-fx / maybe packed NOP)
   if(i<chPattMin[2]) { SWM_Note=(XM_Note!=0) ? (XM_Note>=0x60)?0:XM_Note : GATEOFFX ; }
   else SWM_Note=XM_Note;
   if(SWM_Note!=0 && SWM_Note<0x60) PrevVib=0; //if valid note starts, init Vibrato-signal
   //CASE2 - if there's something to put into instrument-column (instrument/legato/inst-FX) 
   if ( XM_Ins < maxinstamount ) //check if legal instrument-selection happens in XM-pattern
   {
    if ((SWM_Ins=XM_Ins)) //check if SWM_ins and XM_ins is nonzero
    {
     if (SWM_Ins!=PrevInst) { SWM_Ins=XM_Ins; PrevInst=SWM_Ins; } //store actual instrument to check later if repeated
     else SWM_Ins=0; //no need to refresh the same instrument in SWM. can be used for FX/legato instead
    }
   }
   if (SWM_Ins) SWM_Note|=0x80; //switch bit7 on
   //write out note/inst. columns
   PATTERNS[i][SWMpattIndex]=(PrevNote==SWM_Note && i<chPattMin[2]) ? 0 : SWM_Note;  SWMpattIndex++; //note column always exists
   if (SWM_Ins) { PATTERNS[i][SWMpattIndex]=SWM_Ins; SWMpattIndex++; }
   PrevNote=SWM_Note&0x7F;
  }
  PATTERNS[i][SWMpattIndex]=0xFF; SWMpattIndex++; //put $FF pattern-delimiter - will be overwritten by size-indicator
  PATTERNSIZES[i]=SWMpattIndex;
 } 
 //maybe we should look for transposed patterns (where note-differences and effects are the same)
 //(Adjust 'PatternAmount' if less pattern was used by further optimizations)
 for (i=1;i<=PatternAmount;i++) //pack patterns into TUNEDATA[] array
 {
  for (j=PrevData=0;j<PATTERNSIZES[i];j++) 
  {
   PattData=PATTERNS[i][j];
   //pack NOPs if enough found------------------------- (mimic the packdepack.inc assembly pattern-packer algorithm)
   if ( (PattData!=0) || (j==0) ) goto wrcmptr;
   else if (PATTERNS[i][j-1]) goto wrcmptr;
   NOPcount=PACKEDMIN-1; //init
   packloop: if (PATTERNS[i][j+1]!=0) goto putprst; 
   NOPcount++; j++; 
   if ( NOPcount < PACKEDMAX ) goto packloop; 
   putprst: if ((PattData=NOPcount)==(PACKEDMIN-1)) PattData=0;  //simple NOP in case of only 2 consequent zeroes
   //write Patterndata to target-----------------------
   wrcmptr: TUNEDATA[DataIndex]=PrevData=PattData; DataIndex++; 
  }
  //printf("\n");
  printf("  pattern$%X(length:$%2X,size:$%2X),  ",i,PATTERNLENGTHS[i],PATTERNSIZES[i]);
  DataIndex--; TUNEDATA[DataIndex]=PATTERNSIZES[i]; DataIndex++; //write out size of pattern to TUNEDATA - in place of 0xFF delimiter
  TUNEDATA[DataIndex]=PATTERNLENGTHS[i]; DataIndex++; //write length of pattern to TUNEDATA after size
 } 
  
  printf("\n\nWriting SWM instruments to disk:  "); 
  TUNEDATA[DataIndex]=0x00; DataIndex++; //0 CONTROL REGISTER (HR TIMER,HRTYPE ,VIBRATO-TYPE,TIED PW/CTF)
  TUNEDATA[DataIndex]=0x0F;DataIndex++; TUNEDATA[DataIndex]=((XMinsSustain[i]&0x3F)/4)*16+0;DataIndex++; //1..2 HR-ADSR
  TUNEDATA[DataIndex]=0; DataIndex++; //3 Attack/Decay ON NOTESTART (GATE-ON)
  TUNEDATA[DataIndex]=ORMheader[4]&0xF0+4; DataIndex++; //3 Sustain/Release ON NOTESTART (GATE-ON) - force release '4' in SW
  TUNEDATA[DataIndex]=0; DataIndex++; //5 VIBRATO (CALCULATED) FREQ+AMP (amplitude:high nibble)
  TUNEDATA[DataIndex]=0; DataIndex++; //6 VIBRATO DELAY OR AMPLITUDE-INCREMENT SPEED
  TUNEDATA[DataIndex]=0; DataIndex++; //7 ARPEGGIO AND CHORD-SPEED (0=1X)
  TUNEDATA[DataIndex]=1; DataIndex++; //8 DEFAULT CHORD FOR INSTRUMENT
  TUNEDATA[DataIndex]=0; DataIndex++; //round(XMinsTransp[i]/12)*12-12; DataIndex++; //9 OCTAVE SHIFT (2's complement) - it's transpose essentially but in 12 steps by editor
  TUNEDATA[DataIndex]=0x14; DataIndex++; //A PW-TABLE POINTER RELATIVE TO INSTRUMENT-ADDRESS
  TUNEDATA[DataIndex]=0x18; DataIndex++; //B FILTER-TABLE POINTER RELATIVE TO INSTRUMENT-ADDRESS
  TUNEDATA[DataIndex]=0; DataIndex++; //C GATEOFF POINTER FOR WFARP-TABLE RELATIVE TO WF-TABLE POSITION
  TUNEDATA[DataIndex]=0; DataIndex++; //D GATEOFF POINTER FOR PW-TABLE RELATIVE TO PW-TABLE POSITION
  TUNEDATA[DataIndex]=0; DataIndex++; //E GATEOFF POINTER FOR FILTER-TABLE RELATIVE TO FILTER-TABLE POS.
  TUNEDATA[DataIndex]=0; DataIndex++; //F - used as 1st-frame waveform if its switch enabled in control-byte - in SW1.x tracker 00 value gets converted to $09 default
  TUNEDATA[DataIndex]=ORMheader[3]; DataIndex++;TUNEDATA[DataIndex]=0; DataIndex++;TUNEDATA[DataIndex]=0; DataIndex++;TUNEDATA[DataIndex]=0xFF; DataIndex++; //WFtable
  TUNEDATA[DataIndex]=ORMheader[2]|0x80;DataIndex++; TUNEDATA[DataIndex]=0;DataIndex++; TUNEDATA[DataIndex]=0;DataIndex++; //PulseWidth-table
  TUNEDATA[DataIndex]=0xFF; DataIndex++; //PulseWidth-table
  TUNEDATA[DataIndex]=0x18; DataIndex++; // !!!! would be Filter-table 0xFF delimiter, but turns into size-info for SWM module decompression
  printf("\nSWM instrument $1, name: "); char INS_NAM1[]="1R-CHAN1";
  for(j=0;j<instnamelength;j++) { TUNEDATA[DataIndex]=INS_NAM1[j]; printf("%c",TUNEDATA[DataIndex]); DataIndex++; }

  TUNEDATA[DataIndex]=0x00; DataIndex++; //0 CONTROL REGISTER (HR TIMER,HRTYPE ,VIBRATO-TYPE,TIED PW/CTF)
  TUNEDATA[DataIndex]=0x0F;DataIndex++; TUNEDATA[DataIndex]=((XMinsSustain[i]&0x3F)/4)*16+0;DataIndex++; //1..2 HR-ADSR
  TUNEDATA[DataIndex]=0; DataIndex++; //3 Attack/Decay ON NOTESTART (GATE-ON)
  TUNEDATA[DataIndex]=ORMheader[7]&0xF0+4; DataIndex++; //3 Sustain/Release ON NOTESTART (GATE-ON) - force release '4' in SW
  TUNEDATA[DataIndex]=0; DataIndex++; //5 VIBRATO (CALCULATED) FREQ+AMP (amplitude:high nibble)
  TUNEDATA[DataIndex]=0; DataIndex++; //6 VIBRATO DELAY OR AMPLITUDE-INCREMENT SPEED
  TUNEDATA[DataIndex]=0; DataIndex++; //7 ARPEGGIO AND CHORD-SPEED (0=1X)
  TUNEDATA[DataIndex]=1; DataIndex++; //8 DEFAULT CHORD FOR INSTRUMENT
  TUNEDATA[DataIndex]=0; DataIndex++; //round(XMinsTransp[i]/12)*12-12; DataIndex++; //9 OCTAVE SHIFT (2's complement) - it's transpose essentially but in 12 steps by editor
  TUNEDATA[DataIndex]=0x14; DataIndex++; //A PW-TABLE POINTER RELATIVE TO INSTRUMENT-ADDRESS
  TUNEDATA[DataIndex]=0x18; DataIndex++; //B FILTER-TABLE POINTER RELATIVE TO INSTRUMENT-ADDRESS
  TUNEDATA[DataIndex]=0; DataIndex++; //C GATEOFF POINTER FOR WFARP-TABLE RELATIVE TO WF-TABLE POSITION
  TUNEDATA[DataIndex]=0; DataIndex++; //D GATEOFF POINTER FOR PW-TABLE RELATIVE TO PW-TABLE POSITION
  TUNEDATA[DataIndex]=0; DataIndex++; //E GATEOFF POINTER FOR FILTER-TABLE RELATIVE TO FILTER-TABLE POS.
  TUNEDATA[DataIndex]=0; DataIndex++; //F - used as 1st-frame waveform if its switch enabled in control-byte - in SW1.x tracker 00 value gets converted to $09 default
  TUNEDATA[DataIndex]=ORMheader[6]; DataIndex++;TUNEDATA[DataIndex]=0; DataIndex++;TUNEDATA[DataIndex]=0; DataIndex++;TUNEDATA[DataIndex]=0xFF; DataIndex++; //WFtable
  TUNEDATA[DataIndex]=ORMheader[5]|0x80;DataIndex++; TUNEDATA[DataIndex]=0;DataIndex++; TUNEDATA[DataIndex]=0;DataIndex++; //PulseWidth-table
  TUNEDATA[DataIndex]=0xFF; DataIndex++; //PulseWidth-table
  TUNEDATA[DataIndex]=0x18; DataIndex++; // !!!! would be Filter-table 0xFF delimiter, but turns into size-info for SWM module decompression
  printf("\nSWM instrument $2, name: "); char INS_NAM2[]="1R-CHAN2";
  for(j=0;j<instnamelength;j++) { TUNEDATA[DataIndex]=INS_NAM2[j]; printf("%c",TUNEDATA[DataIndex]); DataIndex++; }

 char INS_NAM3[]="1R-PRG00", hexchar[0x10]="0123456789ABCDEF";
 unsigned char ProgTemp[3][0x20],ProgLength; unsigned int ReadPitch;
 for(i=2;i<InstAmount;i++) //generate rest of the instruments based on ORM program-table and Instrument-list array
 {
  for (j=0;j<0x1E;j++)
  {
   for (k=0;k<3;k++) 
   {
    ProgTemp[k][j]=ORMprogTable[k][InstList[i]+j]; 
   }
   if (ProgTemp[2][j]==0xFF)  //end of program - decide if putting loop-command or end-command
   { 
	if (ProgTemp[1][j]!=0 && ProgTemp[1][j]>=InstList[i] && ProgTemp[1][j]<InstList[i]+j) //check if jump-address is in range
	{
	 ProgTemp[0][j]=0xFE; ProgTemp[1][j]=(ProgTemp[1][j]-InstList[i])*3+WFTABLEPOS; ProgTemp[2][j]=0; j++;
    } 
	break;
   }
   else if (ProgTemp[0][j]==0) { ProgTemp[1][j]=0; ProgTemp[2][j]=0; } //don't transfer effects
   else
   {
	ReadPitch=ProgTemp[2][j]*256+ProgTemp[1][j];
	for(l=0;l<=255;l++)
	{
	 if ( FreqTablePalHi[l]*0x100+FreqTablePalLo[l] >= ReadPitch ) break;
    }
    ProgTemp[1][j]=l|0x80; ProgTemp[2][j]=0; //abs.pitch.
   }
   //printf("%.2X-%.2X-%.2X ",ProgTemp[0][j],ProgTemp[1][j],ProgTemp[2][j]);
  }
  ProgLength=j;
	 
  TUNEDATA[DataIndex]=0x08; DataIndex++; //0 CONTROL REGISTER (HR TIMER,HRTYPE ,VIBRATO-TYPE,TIED PW/CTF)
  TUNEDATA[DataIndex]=0x0F;DataIndex++; TUNEDATA[DataIndex]=((XMinsSustain[i]&0x3F)/4)*16+0;DataIndex++; //1..2 HR-ADSR
  TUNEDATA[DataIndex]=ORMheader[9]; DataIndex++; //3 Attack/Decay ON NOTESTART (GATE-ON)
  TUNEDATA[DataIndex]=ORMheader[10]; DataIndex++; //3 Sustain/Release ON NOTESTART (GATE-ON)
  TUNEDATA[DataIndex]=0; DataIndex++; //5 VIBRATO (CALCULATED) FREQ+AMP (amplitude:high nibble)
  TUNEDATA[DataIndex]=0; DataIndex++; //6 VIBRATO DELAY OR AMPLITUDE-INCREMENT SPEED
  TUNEDATA[DataIndex]=0; DataIndex++; //7 ARPEGGIO AND CHORD-SPEED (0=1X)
  TUNEDATA[DataIndex]=1; DataIndex++; //8 DEFAULT CHORD FOR INSTRUMENT
  TUNEDATA[DataIndex]=0; DataIndex++; //round(XMinsTransp[i]/12)*12-12; DataIndex++; //9 OCTAVE SHIFT (2's complement) - it's transpose essentially but in 12 steps by editor
  TUNEDATA[DataIndex]=0x11+ProgLength*3; DataIndex++; //A PW-TABLE POINTER RELATIVE TO INSTRUMENT-ADDRESS
  TUNEDATA[DataIndex]=0x15+ProgLength*3; DataIndex++; //B FILTER-TABLE POINTER RELATIVE TO INSTRUMENT-ADDRESS
  TUNEDATA[DataIndex]=0; DataIndex++; //C GATEOFF POINTER FOR WFARP-TABLE RELATIVE TO WF-TABLE POSITION
  TUNEDATA[DataIndex]=0; DataIndex++; //D GATEOFF POINTER FOR PW-TABLE RELATIVE TO PW-TABLE POSITION
  TUNEDATA[DataIndex]=0; DataIndex++; //E GATEOFF POINTER FOR FILTER-TABLE RELATIVE TO FILTER-TABLE POS.
  TUNEDATA[DataIndex]=9; DataIndex++; //F - used as 1st-frame waveform if its switch enabled in control-byte - in SW1.x tracker 00 value gets converted to $09 default
  for(j=0;j<ProgLength;j++)
  {
   for(k=0;k<3;k++) 
   {
	TUNEDATA[DataIndex]=ProgTemp[k][j];
    DataIndex++; 
   }
  }
  TUNEDATA[DataIndex]=0xFF; DataIndex++; //WFtable-ending
  TUNEDATA[DataIndex]=ORMheader[8]|0x80;DataIndex++; TUNEDATA[DataIndex]=0;DataIndex++; TUNEDATA[DataIndex]=0;DataIndex++; //PulseWidth-table
  TUNEDATA[DataIndex]=0xFF; DataIndex++; //PulseWidth-table
  TUNEDATA[DataIndex]=0x15+ProgLength*3; DataIndex++; // !!!! would be Filter-table 0xFF delimiter, but turns into size-info for SWM module decompression
   printf("\nSWM instrument $%X, name: ",i+1); 
   for(j=0;j<instnamelength-2;j++) { TUNEDATA[DataIndex]=INS_NAM3[j]; printf("%c",TUNEDATA[DataIndex]); DataIndex++; }
   TUNEDATA[DataIndex]=hexchar[InstList[i]/0x10]; printf("%c",TUNEDATA[DataIndex]); DataIndex++; 
   TUNEDATA[DataIndex]=hexchar[InstList[i]&0xF]; printf("%c",TUNEDATA[DataIndex]); DataIndex++; 
 }
 
 //writing SWM subtune-funktempo (only one subtune)
 TUNEDATA[DataIndex]=TUNETEMPO[0][0]|0x80; DataIndex++; //funktempo-program not supported in XM this simple way
 TUNEDATA[DataIndex]=0x84; DataIndex++; //won't be used anyway
 
 OutputSize=DataIndex;
 //write out array TUNEDATA to disk
 //if ( !strcmp(FilExt(OutputFileName),".swm") ) strcat(OutputFileName,".prg"); //if only .swm given, append .prg
 OutputFile = fopen(OutputFileName,"wb"); // !!!important "wb" is for binary (not to check text-control characters)
 LEwordToFile(0x3333); //-2. load-address of PRG format
 fwrite(TUNEDATA,OutputSize,sizeof(unsigned char),OutputFile); 
 printf("\n\n**** Conversion is complete. **** \nSWM file '%s' generated... :)",OutputFileName);
 fclose(OutputFile);
 
 printf("\n-----------------------------------------------------\n");
}


//=================== Common Functions ======================================================
void WaitKeyPress()
{
 //printf("\nPress any key...\n\n");
 //getchar();
}

char* FilExt(char *filename) //get pointer of file-extension from filename string
{  //if no '.' found, point to end of the string
 char* LastDotPos = strrchr(filename,'.');
 if (LastDotPos == NULL) return (filename+strlen(filename)); //make strcmp not to find match, otherwise it would be segmentation fault
 return LastDotPos;
}

void ChangeExt(char *filename,char *newExt) //change the extension of the file 
{
 CutExt(filename); //omit original extension with 0 string-delimiter
 strcat(filename,newExt); //expand with new extension
}

void CutExt(char *filename) //cut the extension of the filename by putting 0 at position of '.'
{
 *FilExt(filename)=0; //omit original extension with 0 string-delimiter
}

unsigned int BEwordFromFile() //read a big-endian word from input-file
{
 return ( (fgetc(InputFile)*256) + fgetc(InputFile) );
}

unsigned long ReadVarLenNum() //read variable-length encoded number from the file
{
 int i=0;
 unsigned char SevenBits, NumberOfDigits; //hopefully no more than 8 digits exist
 unsigned long VarLenNum=0;
 for(i=0; i<MIDI_VLV_LEN_MAX; i++) 
 {
  SevenBits=fgetc(InputFile);
  Digits[i]=SevenBits&0x7F;
  if ((SevenBits&0x80)==0) break;
 }
 NumberOfDigits=i;
 for(i=0; i<=NumberOfDigits; i++)
 {
  VarLenNum+=((int)pow(128.00,(float)(NumberOfDigits-i)) * Digits[i]);
  //printf(" *%x-%x-%x",i,Digits[i],(int)pow(128.00,(float)(NumberOfDigits-i)));
 }
 //printf(" %x*",VarLenNum);
 return VarLenNum;
}

char LowByte(int num) //returns low-byte of integer
{
 return num-(num/256)*256; //integer division - no need for 'floor' function
}

char HiByte(int num) //returns hi-byte of integer
{
 return (num/256); //integer division - no need for 'floor' function
}

int LEwordToFile(unsigned int DataToWrite) //put word in little-endian to output-file
{
 fputc(LowByte(DataToWrite),OutputFile); 
 return fputc(HiByte(DataToWrite),OutputFile); 
}

unsigned int LEwordFromFile() //read a little-endian word from input-file
{
 return ( fgetc(InputFile) + (fgetc(InputFile)*256) );
}

//============================================================================================================
